@@ -1,7 +1,7 @@
 package Module::Install::ForC;
 use strict;
 use warnings;
-our $VERSION = '0.11';
+our $VERSION = '0.20';
 use 5.008000;
 use Module::Install::ForC::Env;
 use Config;              # first released with perl 5.00307
@@ -11,92 +11,55 @@ use FindBin;             # first released with perl 5.00307
 use Module::Install::Base;
 our @ISA     = qw(Module::Install::Base);
 
-our @TARGETS;
 our %OBJECTS;
-our $POSTAMBLE;
-our @TESTS;
-our %INSTALL;
+our @CONFIG_H;
 
 sub env_for_c {
     my $self = shift;
+    $self->_forc_initialize();
     $self->admin->copy_package('Module::Install::ForC::Env');
     Module::Install::ForC::Env->new($self, @_)
 }
 sub is_linux () { $^O eq 'linux'  }
 sub is_mac   () { $^O eq 'darwin' }
 sub is_win32 () { $^O eq 'MSWin32' }
-sub WriteMakefileForC {
-    my $self = shift;
 
-    my $src = $self->_gen_makefile();
+# (DEPRECATED)
+sub WriteMakefileForC { shift->WriteMakefile(@_) }
 
-    open my $fh, '>', 'Makefile' or die "cannot open file: $!";
-    print $fh $src;
+sub WriteHeaderForC {
+    my ($self, $fname) = @_;
+    $fname or die "Usage: WriteHeaderForC('foo_config.h')";
+
+    (my $guard = $fname) =~ tr{a-z./\055}{A-Z___};
+
+    my $header = "#ifndef $guard\n"
+               . "#define $guard\n\n";
+    my $footer = "\n\n#endif  /* $guard */\n";
+    if (my $version = $self->version) {
+        (my $verkey = $self->name) =~ s/^Clib-//;
+        $verkey =~ tr{a-z./\055}{A-Z___};
+        $verkey .= "_VERSION";
+        $header .= qq{#define $verkey "$version"\n\n};
+    }
+
+    open my $fh, '>', $fname or die "cannot open file($fname): $!";
+    print $fh $header . join('', @CONFIG_H) . $footer;
     close $fh;
 }
 
-sub _gen_makefile {
-    my $self = shift;
-    $self->name(File::Basename::basename($FindBin::Bin)) unless $self->name;
-    $self->version('') unless defined $self->version;
+{
+    my $initialized = 0;
+    sub _forc_initialize {
+        return if $initialized++;
 
-    my $mm = ExtUtils::MM->new(
-        {
-            NAME         => $self->name,
-            VERSION      => $self->version,
-        }
-    );
-    my $mm_params = join("\n", map { $_.'='.($mm->{$_} || '') } qw/FIRST_MAKEFILE MOD_INSTALL ABSPERL ABSPERLRUN VERBINST UNINST PERM_DIR PERL PREOP TRUE TAR RM_F RM_RF NOECHO NOOP INSTALLARCHLIB INSTALL_BASE DIST_CP DIST_DEFAULT POSTOP COMPRESS TARFLAGS TO_UNIX PERLRUN DISTVNAME VERSION NAME ECHO ECHO_N MAKE MV SUFFIX ZIP SHAR FULLPERLRUN FULLPERL USE_MAKEFILE FIXIN DOC_INSTALL UNINSTALL CP INST_LIB INST_ARCHLIB/);
-    (my $make = <<"...") =~ s/^[ ]{4}/\t/gmsx;
-$mm_params
-TEST_VERBOSE=0
-TEST_FILES=@{[ $self->tests || '' ]}
-
-.PHONY: all config static dynamic test linkext manifest blibdirs clean realclean disttest distdir
-
-@{[ $mm->special_targets ]}
-
-all: @Module::Install::ForC::TARGETS
-
-config :: \$(FIRST_MAKEFILE)
-    \$(NOECHO) \$(NOOP)
-
-test: @TESTS
-    @{[ $mm->test_via_harness('$(FULLPERLRUN)', '$(TEST_FILES)') ]}
-
-dist: \$(DIST_DEFAULT) \$(FIRST_MAKEFILE)
-
-clean:
-	\$(RM_F) @Module::Install::ForC::TARGETS @{[ keys %Module::Install::ForC::OBJECTS ]}
-
-realclean :: clean
-	\$(RM_F) Makefile
-    \$(RM_RF) \$(DISTVNAME)
-	@{[ $Config{rm_try} || '' ]}
-
-install: all config
-	@{[ join("\n\t", map { @{ $_ } } values %Module::Install::ForC::INSTALL) ]}
-    \$(NOECHO) \$(NOOP)
-
-@{[ $mm->metafile_target ]}
-
-@{[ $mm->distmeta_target ]}
-
-@{[ $mm->distdir ]}
-
-@{[ $mm->dist_test ]}
-
-# dist_basics
-@{[ $mm->dist_basics ]}
-
-# dist_core
-@{[ $mm->dist_core ]}
-
-@{[ $Module::Install::ForC::POSTAMBLE || '' ]}
-
-@{[ $self->postamble || '' ]}
-...
-    $make;
+        my $self = shift;
+        $self->makemaker_args(
+            # linking, compiling is job for ForC.
+            C      => [],
+            OBJECT => '',
+        );
+    }
 }
 
 1;
@@ -114,7 +77,7 @@ Module::Install::ForC - the power of M::I for C programs
     my $env = env_for_c(CPPPATH => ['picoev/', 'picohttpparser/']);
     $env->program('testechoclient' => ["testechoclient.c"]);
 
-    WriteMakefileForC();
+    WriteMakefile();
 
     # then, you will get the Makefile:
     all: testechoclient
@@ -154,9 +117,9 @@ This is a early BETA release! API will change later.
 
 Is this the OS or not?
 
-=item WriteMakefileForC()
+=item WriteHeaderForC("config.h")
 
-Write makefile in M::I::ForC style.
+Write config.h, contains HAVE_* style definitions generated by $env->have_header, $env->have_library.
 
 =item my $env = env_for_c(CPPPATH => ['picoev/', 'picohttpparser/']);
 
